-- Update Job table --
ALTER TABLE IF EXISTS Job ADD COLUMN IF NOT EXISTS operation_count INTEGER;
ALTER TABLE IF EXISTS Job ADD COLUMN IF NOT EXISTS last_updated TIMESTAMP WITH TIME ZONE;

-- Update Operation table --
ALTER TABLE IF EXISTS Operation ADD COLUMN IF NOT EXISTS last_updated TIMESTAMP WITH TIME ZONE;

-- Create Platform enum --
CREATE TABLE IF NOT EXISTS Platform (
    id SMALLINT PRIMARY KEY,
    name VARCHAR(32) NOT NULL
);

INSERT INTO Platform(id, name) VALUES (0, 'UNIVERSAL') ON CONFLICT DO NOTHING;
INSERT INTO Platform(id, name) VALUES (1, 'WINDOWS') ON CONFLICT DO NOTHING;
INSERT INTO Platform(id, name) VALUES (2, 'MAC') ON CONFLICT DO NOTHING;
INSERT INTO Platform(id, name) VALUES (3, 'LINUX') ON CONFLICT DO NOTHING;

-- Create Architecture enum --
CREATE TABLE IF NOT EXISTS Architecture (
    id SMALLINT PRIMARY KEY,
    name VARCHAR(32) NOT NULL
);

INSERT INTO Architecture(id, name) VALUES (0, 'UNIVERSAL') ON CONFLICT DO NOTHING;
INSERT INTO Architecture(id, name) VALUES (1, 'BIT32') ON CONFLICT DO NOTHING;
INSERT INTO Architecture(id, name) VALUES (2, 'BIT64') ON CONFLICT DO NOTHING;
INSERT INTO Architecture(id, name) VALUES (3, 'ARM64') ON CONFLICT DO NOTHING;

-- Create Distribution enum --
CREATE TABLE IF NOT EXISTS Distribution (
    id SMALLINT PRIMARY KEY,
    name VARCHAR(32) NOT NULL
);

INSERT INTO Distribution(id, name) VALUES (0, 'UNIVERSAL') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (1, 'AMAZON_LINUX') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (2, 'RHEL_CENTOS') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (3, 'UBUNTU') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (4, 'SUSE') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (5, 'AMAZON') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (6, 'RHEL') ON CONFLICT DO NOTHING;
INSERT INTO Distribution(id, name) VALUES (7, 'CENTOS') ON CONFLICT DO NOTHING;

-- Update Sensor table --
ALTER TABLE IF EXISTS Sensor ADD COLUMN IF NOT EXISTS platform SMALLINT;
ALTER TABLE IF EXISTS Sensor ADD COLUMN IF NOT EXISTS architecture SMALLINT;
ALTER TABLE IF EXISTS Sensor ADD COLUMN IF NOT EXISTS distribution SMALLINT;

CREATE TABLE IF NOT EXISTS JobDelivery(
    id BIGSERIAL PRIMARY KEY,
    job UUID references Job(id) ON DELETE CASCADE NOT NULL,
    platform SMALLINT references Platform(id) NOT NULL,
    architecture SMALLINT references Architecture(id) NOT NULL,
    distribution SMALLINT references Distribution(id) NOT NULL,
    group_name VARCHAR(255) NOT NULL,
    group_id UUID NOT NULL,
    segment_ext_urn VARCHAR(255) NOT NULL,
    filename VARCHAR(255) NOT NULL,
    content_hash VARCHAR(255) NOT NULL,
    UNIQUE(job, segment_ext_urn)
);

CREATE INDEX IF NOT EXISTS idx_jobdelivery_job_segment_lookup ON JobDelivery (job, segment_ext_urn);

-- Create JobAction enum --
CREATE TABLE IF NOT EXISTS JobActionType (
    id SMALLINT PRIMARY KEY,
    name VARCHAR(32) NOT NULL
);

INSERT INTO JobActionType(id, name) VALUES (0, 'STOP') ON CONFLICT DO NOTHING;

-- Create JobActionStatus enum --
CREATE TABLE IF NOT EXISTS JobActionStatus (
     id SMALLINT PRIMARY KEY,
     name VARCHAR(32) NOT NULL
);

INSERT INTO JobActionStatus(id, name) VALUES (0, 'UNPROCESSED') ON CONFLICT DO NOTHING;
INSERT INTO JobActionStatus(id, name) VALUES (1, 'PROCESSING') ON CONFLICT DO NOTHING;
INSERT INTO JobActionStatus(id, name) VALUES (2, 'COMPLETED') ON CONFLICT DO NOTHING;

CREATE TABLE IF NOT EXISTS JobAction(
    id BIGSERIAL PRIMARY KEY,
    job UUID references Job(id) ON DELETE CASCADE NOT NULL,
    action SMALLINT references JobActionType(id) NOT NULL,
    status SMALLINT references JobActionStatus(id) NOT NULL,
    UNIQUE(job, action)
);

CREATE INDEX IF NOT EXISTS idx_jobaction_job_status_lookup ON JobAction (job, status);

CREATE TABLE IF NOT EXISTS JobFeed(
    id BIGSERIAL PRIMARY KEY,
    job UUID references Job(id) ON DELETE CASCADE NOT NULL,
    last_generated TIMESTAMP WITH TIME ZONE NOT NULL,
    UNIQUE(job)
);

CREATE INDEX IF NOT EXISTS idx_jobfeed_job_lookup ON JobFeed (job);
