CREATE EXTENSION IF NOT EXISTS "uuid-ossp";
CREATE TABLE IF NOT EXISTS Organization(
   id UUID PRIMARY KEY DEFAULT uuid_generate_v1mc(),
   org_key VARCHAR(50) NOT NULL UNIQUE,
   last_processed TIMESTAMP WITH TIME ZONE,
   processing_lock BOOLEAN DEFAULT FALSE NOT NULL,
   max_concurrent_operations INT NOT NULL
);
CREATE INDEX idx_organization_org_key ON Organization (org_key);
CREATE INDEX idx_organization_updated_at ON Organization (last_processed);
CREATE TABLE IF NOT EXISTS Job (
    id UUID PRIMARY KEY DEFAULT uuid_generate_v1mc(),
    created_at TIMESTAMP WITH TIME ZONE NOT NULL,
    started_at TIMESTAMP WITH TIME ZONE,
    completed_at TIMESTAMP WITH TIME ZONE,
    organization UUID references Organization(id) NOT NULL,
    status VARCHAR(32) DEFAULT 'PENDING' NOT NULL,
    tag VARCHAR(32) DEFAULT 'ENDPOINT' NOT NULL,
    login_name VARCHAR(254) NOT NULL,
    client_ip VARCHAR(45) NOT NULL
);
CREATE INDEX idx_job_organization ON Job (organization);
CREATE INDEX idx_job_status ON Job (status);
CREATE INDEX idx_job_tag ON Job (tag);
CREATE TABLE IF NOT EXISTS SensorVersion(
   id UUID PRIMARY KEY DEFAULT uuid_generate_v1mc(),
   type VARCHAR(32) NOT NULL,
   version VARCHAR(254) NOT NULL,
   UNIQUE(type, version)
);
CREATE TABLE IF NOT EXISTS Sensor (
   id UUID PRIMARY KEY DEFAULT uuid_generate_v1mc(),
   device_id INT NOT NULL UNIQUE,
   current_version UUID references SensorVersion(id) NOT NULL,
   target_version UUID references SensorVersion(id) NOT NULL,
   operation_lock BOOLEAN DEFAULT FALSE NOT NULL
);
CREATE INDEX idx_sensor_device_id ON Sensor (device_id);
CREATE TABLE IF NOT EXISTS Operation(
   id UUID PRIMARY KEY DEFAULT uuid_generate_v1mc(),
   job UUID references Job(id) NOT NULL,
   sensor UUID references Sensor(id) NOT NULL,
   status VARCHAR(32) DEFAULT 'PENDING' NOT NULL,
   initial_version UUID references SensorVersion(id),
   requested_version UUID references SensorVersion(id),
   performed_at TIMESTAMP WITH TIME ZONE,
   completed_at TIMESTAMP WITH TIME ZONE,
   UNIQUE(job, sensor)
);
CREATE INDEX idx_operation_job ON Operation (job);
CREATE INDEX idx_operation_status ON Operation (status);
CREATE TABLE IF NOT EXISTS Job_SensorVersion (
    job UUID references Job(id) NOT NULL,
    version UUID references SensorVersion(id) NOT NULL,
    PRIMARY KEY (job, version)
);